@@ -13,7 +13,7 @@
 ##############################################################################
 """Form and Widget Framework Interfaces
 
-$Id: interfaces.py 118858 2010-12-14 09:36:13Z rogerineichen $
+$Id: interfaces.py 128150 2012-10-30 09:35:25Z malthe $
 """
 __docformat__ = "reStructuredText"
 import zope.i18nmessageid
@@ -47,10 +47,10 @@ class IFormLayer(zope.interface.Interface):
 
     It is intended that someone can just use this layer as a base layer when
     using this package.
-    
+
     Since version 2.4.2, this layer doesn't provide IBrowserRequst anymore.
     This makes it possible to use the IFormLayer within z3c.jsonrpc without
-    to apply the IBrowserRequest into the jsonrpc request. 
+    to apply the IBrowserRequest into the jsonrpc request.
     """
 
 
@@ -114,7 +114,7 @@ class IData(zope.interface.Interface):
 class IValidator(zope.interface.Interface):
     """A validator for a particular value."""
 
-    def validate(value):
+    def validate(value, force=False):
         """Validate the value.
 
         If successful, return ``None``. Otherwise raise an ``Invalid`` error.
@@ -211,6 +211,13 @@ class IField(zope.interface.Interface):
         default=None,
         missing_value=None)
 
+    showDefault = zope.schema.Bool(
+        title=_('Show default value'),
+        description=_('A flag, when set, makes the widget to display'
+                      'field|adapter provided default values.'),
+        default=True,
+        required=False)
+
 
 class IFields(ISelectionManager):
     """IField manager."""
@@ -372,6 +379,16 @@ class ISubformFactory(zope.interface.Interface):
         """Return a default object created to be populated.
         """
 
+# ----[ Widget layout template ]----------------------------------------------
+
+class IWidgetLayoutTemplate(zope.interface.Interface):
+    """Widget layout template marker used for render the widget layout.
+
+    It is important that we don't inherit this template from IPageTemplate.
+    otherwise we will get into trouble since we lookup an IPageTemplate
+    in the widget/render method.
+
+    """
 
 # ----[ Widgets ]------------------------------------------------------------
 
@@ -420,6 +437,7 @@ class IWidget(ILocation):
         required=False)
 
     template = zope.interface.Attribute('''The widget template''')
+    layout = zope.interface.Attribute('''The widget layout template''')
 
     ignoreRequest = zope.schema.Bool(
         title=_('Ignore Request'),
@@ -428,7 +446,7 @@ class IWidget(ILocation):
         default=False,
         required=False)
 
-    #ugly thing to remove setErrors parameter from extract
+    # ugly thing to remove setErrors parameter from extract
     setErrors = zope.schema.Bool(
         title=_('Set errors'),
         description=_('A flag, when set, the widget sets error messages '
@@ -436,6 +454,22 @@ class IWidget(ILocation):
         default=True,
         required=False)
 
+    # a bit different from ignoreRequiredOnExtract, because we record
+    # here the fact, but for IValidator, because the check happens there
+    ignoreRequiredOnValidation = zope.schema.Bool(
+        title=_('Ignore Required validation'),
+        description=_("If set then required fields will pass validation "
+                      "regardless whether they're filled in or not"),
+        default=False,
+        required=True)
+
+    showDefault = zope.schema.Bool(
+        title=_('Show default value'),
+        description=_('A flag, when set, makes the widget to display'
+                      'field|adapter provided default values.'),
+        default=True,
+        required=False)
+
     def extract(default=NO_VALUE):
         """Extract the string value(s) of the widget from the form.
 
@@ -456,7 +490,10 @@ class IWidget(ILocation):
         """Setup all of the widget information used for displaying."""
 
     def render():
-        """Return the widget's text representation."""
+        """Render the plain widget without additional layout"""
+
+    def __call__():
+        """Render a layout template which is calling widget/render"""
 
 
 class ISequenceWidget(IWidget):
@@ -611,6 +648,13 @@ class IWidgets(IManager):
         default=False,
         required=True)
 
+    ignoreRequiredOnExtract = zope.schema.Bool(
+        title=_('Ignore Required validation on extract'),
+        description=_("If set then required fields will pass validation "
+                      "on extract regardless whether they're filled in or not"),
+        default=False,
+        required=True)
+
     hasRequiredFields = zope.schema.Bool(
         title=_('Has required fields'),
         description=_('A flag set when at least one field is marked as '
@@ -874,6 +918,13 @@ class IForm(zope.interface.Interface):
         default=False,
         required=True)
 
+    ignoreRequiredOnExtract = zope.schema.Bool(
+        title=_('Ignore Required validation on extract'),
+        description=_("If set then required fields will pass validation "
+                      "on extract regardless whether they're filled in or not"),
+        default=False,
+        required=True)
+
     widgets = zope.schema.Object(
         title=_('Widgets'),
         description=_('A widget manager containing the widgets to be used in '
@@ -906,11 +957,14 @@ class IForm(zope.interface.Interface):
     def getContent():
         '''Return the content to be displayed and/or edited.'''
 
-    def updateWidgets():
+    def updateWidgets(prefix=None):
         '''Update the widgets for the form.
 
         This method is commonly called from the ``update()`` method and is
         mainly meant to be a hook for subclasses.
+
+        Note that you can pass an argument for ``prefix`` to override
+        the default value of ``"widgets."``.
         '''
 
     def extractData(setErrors=True):
@@ -1036,7 +1090,7 @@ class IButtonForm(IForm):
 class IGroup(IForm):
     """A group of fields/widgets within a form."""
 
-class IGroupForm(object):
+class IGroupForm(IForm):
     """A form that supports groups."""
 
     groups = zope.schema.Tuple(
@@ -1059,3 +1113,4 @@ class IWidgetEvent(zope.interface.Interface):
 
 class IAfterWidgetUpdateEvent(IWidgetEvent):
     """An event sent out after the widget was updated."""
+
