@@ -13,7 +13,7 @@
 ##############################################################################
 """Widget Framework Implementation
 
-$Id: widget.py 123178 2011-10-29 20:58:58Z rogerineichen $
+$Id: widget.py 127740 2012-09-06 09:19:58Z adamg $
 """
 __docformat__ = "reStructuredText"
 
@@ -51,8 +51,12 @@ class Widget(zope.location.Location):
     error = FieldProperty(interfaces.IWidget['error'])
     value = FieldProperty(interfaces.IWidget['value'])
     template = None
+    layout = None
     ignoreRequest = FieldProperty(interfaces.IWidget['ignoreRequest'])
+    ignoreRequiredOnValidation = FieldProperty(
+        interfaces.IWidget['ignoreRequiredOnValidation'])
     setErrors = FieldProperty(interfaces.IWidget['setErrors'])
+    showDefault = FieldProperty(interfaces.IWidget['showDefault'])
 
     # The following attributes are for convenience. They are declared in
     # extensions to the simple widget.
@@ -106,12 +110,14 @@ class Widget(zope.location.Location):
             # that requires fixing zope.schema first
             if ((value is self.field.missing_value or
                  value is interfaces.NO_VALUE) and
-                self.field.default is not None):
+                self.field.default is not None and
+                self.showDefault):
                 value = self.field.default
                 lookForDefault = True
         # Step 1.3: If we still have not found a value, then we try to get it
         #           from an attribute value
-        if value is interfaces.NO_VALUE or lookForDefault:
+        if ((value is interfaces.NO_VALUE or lookForDefault)
+            and self.showDefault):
             adapter = zope.component.queryMultiAdapter(
                 (self.context, self.request, self.form, self.field, self),
                 interfaces.IValue, name='default')
@@ -131,8 +137,12 @@ class Widget(zope.location.Location):
                 if value is not None:
                     setattr(self, attrName, value.get())
 
-    def render(self):
+    def extract(self, default=interfaces.NO_VALUE):
         """See z3c.form.interfaces.IWidget."""
+        return self.request.get(self.name, default)
+
+    def render(self):
+        """Render the plain widget without additional layout"""
         template = self.template
         if template is None:
             template = zope.component.getMultiAdapter(
@@ -140,9 +150,14 @@ class Widget(zope.location.Location):
                 IPageTemplate, name=self.mode)
         return template(self)
 
-    def extract(self, default=interfaces.NO_VALUE):
-        """See z3c.form.interfaces.IWidget."""
-        return self.request.get(self.name, default)
+    def __call__(self):
+        """Get and return layout template which is calling widget/render"""
+        layout = self.layout
+        if layout is None:
+            layout = zope.component.getMultiAdapter(
+                (self.context, self.request, self.form, self.field, self),
+                interfaces.IWidgetLayoutTemplate, name=self.mode)
+        return layout(self)
 
     def __repr__(self):
         return '<%s %r>' % (self.__class__.__name__, self.name)
@@ -178,7 +193,12 @@ class SequenceWidget(Widget):
             # Ignore no value entries. They are in the request only.
             if token == self.noValueToken:
                 continue
-            term = self.terms.getTermByToken(token)
+            try:
+                term = self.terms.getTermByToken(token)
+            except LookupError:
+                # silently ignore missing tokens, because INPUT_MODE and
+                # HIDDEN_MODE does that too
+                continue
             if zope.schema.interfaces.ITitledTokenizedTerm.providedBy(term):
                 value.append(translate(
                     term.title, context=self.request, default=term.title))
@@ -202,7 +222,7 @@ class SequenceWidget(Widget):
     def extract(self, default=interfaces.NO_VALUE):
         """See z3c.form.interfaces.IWidget."""
         if (self.name not in self.request and
-            self.name+'-empty-marker' in self.request):
+            self.name + '-empty-marker' in self.request):
             return []
         value = self.request.get(self.name, default)
         if value != default:
@@ -451,6 +471,25 @@ class WidgetTemplateFactory(object):
         return self.template
 
 
+class WidgetLayoutFactory(object):
+    """Widget layout template factory."""
+
+    def __init__(self, filename, contentType='text/html',
+                 context=None, request=None, view=None,
+                 field=None, widget=None):
+        self.template = ViewPageTemplateFile(filename, content_type=contentType)
+        zope.component.adapter(
+            util.getSpecification(context),
+            util.getSpecification(request),
+            util.getSpecification(view),
+            util.getSpecification(field),
+            util.getSpecification(widget))(self)
+        zope.interface.implementer(interfaces.IWidgetLayoutTemplate)(self)
+
+    def __call__(self, context, request, view, field, widget):
+        return self.template
+
+
 class WidgetEvent(object):
     zope.interface.implements(interfaces.IWidgetEvent)
 
